#
#	HDML.pm
#		(c)2001-2002 MAKINA/Shuhei Nomura
#		maki@tk6.org1.com http://www2.to/iik/
#
package HDML;
use 5.005;
use strict;
use diagnostics;

$HDML::VERSION = '0.55';

sub new{ 'HDML'; }
sub a{ shift; HDMLa -> tag( @_ ); }
sub action{ shift; HDMLaction -> tag( @_ ); }
sub br{ "<BR>\n"; }
sub ce{ shift; HDMLce -> tag( @_ ); }
sub center{ shift; HDMLcenter -> tag( @_ ); }
sub choice{ shift; HDMLchoice -> tag( @_ ); }
sub display{ shift; HDMLdisplay -> tag( @_ ); }
sub entry{ shift; HDMLentry -> tag( @_ ); }
sub hdml{ shift; HDMLhdml -> tag( @_ ); }
sub img{ shift; HDMLimg -> tag( @_ ); }
sub line{ shift; HDMLline -> tag( @_ ); }
sub nodisplay{ shift; HDMLnodisplay -> tag( @_ ); }
sub right{ shift; HDMLright -> tag( @_ ); }
sub tab{ shift; HDMLtab -> tag( @_ ); }
sub wrap{ "<WRAP>\n"; }
sub true{ HDMLBase::true( @_ ); }

package HDMLa;
@HDMLa::ISA = qw( HDMLBase );

sub info{ {
	name => 'A',
	reserved => qr/accept-charset|accesskey|cancel|clear|dest|friend|label|next|number|postdata|postmethod|receive|rel|retvals|sendreferer|task|vars/,
	boolean => qr/clear|friend|sendreferer/,
}; }

package HDMLaction;
@HDMLaction::ISA = qw( HDMLBase );

sub info{ {
	name => 'ACTION',
	reserved => qr/accept-charset|cancel|clear|dest|friend|icon|label|next|number|postdata|postmethod|receive|rel|retvals|sendreferer|src|task|type|vars/,
	boolean => qr/clear|friend|sendreferer/,
}; }

package HDMLce;
@HDMLce::ISA = qw( HDMLBase );

sub info{ {
	name => 'CE',
	reserved => qr/accept-charset|cancel|clear|dest|friend|label|next|number|postdata|postmethod|receive|rel|retvals|sendreferer|task|value|vars/,
	boolean => qr/clear|friend|sendreferer/,
}; }

package HDMLcenter;
@HDMLcenter::ISA = qw( HDMLBase );

sub info{ { name => 'CENTER' }; }

package HDMLchoice;
@HDMLchoice::ISA = qw( HDMLBase );

sub info{ {
	name => 'CHOICE',
	reserved => qr/bookmark|default|idefault|ikey|key|markable|method|name|title/,
	boolean => qr/markable/,
}; }

package HDMLdisplay;
@HDMLdisplay::ISA = qw( HDMLBase );

sub info{ {
	name => 'DISPLAY',
	reserved => qr/bookmark|markable|name|timerdefault|title/,
	boolean => qr/markable/,
}; }

package HDMLentry;
@HDMLentry::ISA = qw( HDMLBase );

sub info{ {
	name => 'ENTRY',
	reserved => qr/bookmark|emptyok|format|key|markable|name|noecho|title/,
	boolean => qr/emptyok|markable|noecho/,
}; }

package HDMLhdml;
@HDMLhdml::ISA = qw( HDMLBase );

sub info{ {
	name => 'HDML',
	reserved => qr/accessdomain|accesspath|markable|public|ttl|version/,
	boolean => qr/markable|public/,
	default => { version => '3.1' }
}; }

package HDMLimg;
@HDMLimg::ISA = qw( HDMLBase );

sub info{ {
	name => 'IMG',
	reserved => qr/alt|icon|src/,
	default => { alt => '' }
}; }

package HDMLline;
@HDMLline::ISA = qw( HDMLBase );

sub info{ { name => 'LINE' }; }

package HDMLnodisplay;
@HDMLnodisplay::ISA = qw( HDMLBase );

sub info{ {
	name => 'NODISPLAY',
	reserved => qr/name/,
}; }

package HDMLright;
@HDMLright::ISA = qw( HDMLBase );

sub info{ { name => 'RIGHT' }; }

package HDMLtab;
@HDMLtab::ISA = qw( HDMLBase );

sub info{ { name => 'TAB' }; }

package HDMLBase;

my $empty = 'action|br|img|wrap';
my $head = 'ce|center|line|right|tab';

sub tag{
	my $self = shift;
	my $info = $self -> info;
	my $reserved = exists( $info -> { reserved } ) ? $info -> { reserved } : undef;
	my $boolean = exists( $info -> { boolean } ) ? $info -> { boolean } : undef;
	my $default = exists( $info -> { default } ) ? $info -> { default } : {};
	my $parameters = shift;
	my $res = sprintf "<%s", $info -> { name };
	my( $body, $paramlist );
	$parameters = '' unless defined( $parameters );
	if( ref( $parameters ) eq 'HASH' ){
		$parameters = [ %$parameters ];
		for( my $i = 0; $i < @$parameters; $i += 2 ){ $parameters -> [ $i ] =~ s/^-//; }
		$parameters = { @$parameters };
		$body = join '', @_;
		$body = '' unless( defined $body );
	}
	else{
		$body = $parameters;
		$body .= join '', @_;
		$parameters = {};
	}
	$paramlist = [ keys %$default ];
	foreach( @$paramlist ){ $parameters -> { $_ } = $default -> { $_ } unless( exists( $parameters -> { $_ } ) ); }
	$paramlist = [ keys %$parameters ];
	if( defined $reserved ){
		foreach( @$paramlist ){
			if( /^$reserved/i ){
				$parameters -> { $_ } = &true( $parameters -> { $_ } ) if( defined( $boolean ) and /^$boolean/i and ( not $parameters -> { $_ } =~ /[^0-9]/ ) );
				$res .= sprintf( " %s=\"%s\"", "\U$_", $parameters -> { $_ } );
			}
		}
	}
	$res .= ( $info -> { name } =~ /^$head/i ) ? ">$body\n" : ( ( $info -> { name } =~ /^$empty/i ) ? ">\n" : sprintf( ">\n$body\n</%s>\n", $info -> { name } ) );
	return $res;
}
sub true{
	my $value = shift;
	$value = shift while( ref $value );
	return 0 unless( defined $value );
	return 1 if $value =~ /^t/gi;
	return 'FALSE' unless $value =~ /[^0\-]/g;
	return 'TRUE' unless $value =~ /[^1-9\-]/g;
	return 0;
}
1;

__END__

=pod

=head1 NAME

HDML - HDML

=head1 EXAMPLE

	use 5.005;
	use strict;
	use HDML;
	
	my $hdml = HDML -> new;
	print $hdml -> hdml(
		{ -version => '3.0', -public => 1, -markable => 'true' },
		$hdml -> display(
			{ -title => 'Test Card - 1' },
			$hdml -> action( { -type => 'accept', -task => 'gosub', -dest => '#2' } ),
			'FooFoo'
		),
		$hdml -> choice(
			{ -name => '2' },
			$hdml -> line( 'BazBazBazBazBazBaz', $hdml -> wrap ),
			"Bar\n",
			$hdml -> ce( { -value => 'one' }, '1' ),
			$hdml -> ce( { -value => 'two' }, '2' ),
			$hdml -> ce( { -value => 'three' }, '3' )
		)
	);
	exit;

=head1 DESCRIPTION

perlœIHDML( Handbeld Device Markup Language )𐶐郂W[łB
F<CGI.pm>HTML𐶐悤ȊoŊȒPHDML邱Ƃo܂B

=head1 NOTICE

_oO͔oĂ܂񂪗p҂̎gɂĂ͕ss邱Ƃ܂B

Ɗȗłӏssȉӏꍇ񍐂ƂꂵłB
̑ӌv]Ȃǂ <maki@tk6.org1.com> ܂ŁB

=head1 HISTORY

=head2 Version 0.55 (2002/02/26)

=over

=item *
Ŝēxč\zBpbP[W𕪊ʂȃR[hrB

=item *
CEECENTERELINEERIGHTETAB^O̐擪ɎIɓsȂB

=item *
AEACTIONECEECHOICEEDISPLAYENODISPLAYɏI^OsvȂ̂ɕtKvȂ̂ɏȂꂽ肷oOCB

=item *
HDML^OTTLvfoOCB

=item *
vfȗĖ{Ƃēnƍŏ̗vf󗝂ȂoOCB

=back

=head2 Version 0.25 (2002/02/25)

=over

=item *
CE\bhACTIONoOCB

=item *
CENTERELINEERIGHTETAB\bhɕ͂nȂƃG[ƂȂoOCB

=item *
̒l̐擪Ɂu-vĂ폜ȂȂB

=item *
ƂĔzw肵ꍇŏ̗vfƂȂĂvfŜ1̕ƂĔF悤ɂȂB

=back

=head2 Version 0.22 (2002/02/23)

=over

=item *
1č\zBʓIHDML悤ɂȂB

=back

=head2 Version 0.04 (2002/02/21)

=over

=item *
fbLƃJ[h`ł悤ɂȂ

=back

=head1 AUTHOR

(c)2001-2002 MAKINA/Shuhei Nomura <maki@tk6.org1.com>

HDML.pm is GPL software.

=cut
